<?php
class ConnectionManager
{
	/**
	 * Cached Connection objects
	 * @type Array
	 */
	protected $cache = array();

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsData;

	/**
	 * Project connections data
	 * @type Array
	 */
	protected $_connectionsIdByName = array();

	
	/**
	 * An array storing the correspondence between project
	 * datasource tables names and connections ids
	 * @type Array
	 */	
	protected $_tablesConnectionIds;
	
	
	/**
	 * @constructor
	 */
	function ConnectionManager()
	{
		$this->_setConnectionsData();
		$this->_setTablesConnectionIds();
	}
	
	/**
	 * Get connection object by the table name
	 * @param String tName
	 * @return Connection
	 */
	public function byTable( $tName )
	{
		$connId = $this->_tablesConnectionIds[ $tName ];
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}

	/**
	 * Get connection object by the connection name
	 * @param String connName
	 * @return Connection
	 */	
	public function byName( $connName )
	{
		$connId = $this->getIdByName( $connName );
		if( !$connId )
			return $this->getDefault();
		return $this->byId( $connId );
	}
	
	/**
	 * Get connection id by the connection name
	 * @param String connName
	 * @return String
	 */	
	protected function getIdByName( $connName )
	{
		return $this->_connectionsIdByName[ $connName ];
	}
	
	/**
	 * Get connection object by the connection id 
	 * @param String connId
	 * @return Connection
	 */	
	public function byId( $connId )
	{
		if( !isset( $this->cache[ $connId ] ) )
			$this->cache[ $connId ] = $this->getConnection( $connId );

		return $this->cache[ $connId ];
	}
	
	/**
	 * Get the default db connection class
	 * @return Connection
	 */
	public function getDefault()
	{
		return $this->byId( "1188mall_at_localhost" );
	}

	/**
	 * Get the users table db connection 
	 * @return Connection
	 */	
	public function getForLogin()
	{
		return $this->byId( "1188mall_at_localhost" );
	}
	
	/**
	 * Get the log table db connection 
	 * @return Connection
	 */	
	public function getForAudit()
	{
		return $this->getDefault();
	}
	
	/**
	 * Get the locking table db connection 
	 * @return Connection
	 */		
	public function getForLocking()
	{
		return $this->getDefault();
	}	
	
	/**
	 * Get the 'ug_groups' table db connection 
	 * @return Connection
	 */	
	public function getForUserGroups()
	{
		return $this->getDefault();
	}		

	/**
	 * Get the saved searches table db connection 
	 * @return Connection
	 */	
	public function getForSavedSearches()
	{
		return $this->getDefault();
	}

	/**
	 * Get the webreports tables db connection 
	 * @return Connection
	 */		
	public function getForWebReports()
	{
		return $this->getDefault();
	}
	
	/**
	 * @param String connId
	 * @return Connection
	 */
	protected function getConnection( $connId )
	{
		include_once getabspath("connections/Connection.php");
		
		$data = $this->_connectionsData[ $connId ];	
		switch( $data["connStringType"] )
		{
			case "mysql":
				if( useMySQLiLib() )
				{
					include_once getabspath("connections/MySQLiConnection.php");
					return new MySQLiConnection( $data );
				}
				
				include_once getabspath("connections/MySQLConnection.php");	
				return new MySQLConnection( $data );	

			case "mssql":
			case "compact":
				if( useMSSQLWinConnect() )
				{
					include_once getabspath("connections/MSSQLWinConnection.php");
					return new MSSQLWinConnection( $data );
				}
				if( isSqlsrvExtLoaded() )
				{
					include_once getabspath("connections/MSSQLSrvConnection.php");	
					return new MSSQLSrvConnection( $data );
				}
				
				include_once getabspath("connections/MSSQLUnixConnection.php");
				return new MSSQLUnixConnection( $data );			

			case "msaccess":
			case "odbc":
			case "odbcdsn":
			case "custom":
			case "file":
				if( stripos($data["ODBCString"], 'Provider=') !== false )
				{
					include_once getabspath("connections/ADOConnection.php");
					return new ADOConnection( $data );
				}
				
				include_once getabspath("connections/ODBCConnection.php");
				return new ODBCConnection( $data );
			
			case "oracle":
				include_once getabspath("connections/OracleConnection.php");
				return new OracleConnection( $data );

			case "postgre":
				include_once getabspath("connections/PostgreConnection.php");
				return new PostgreConnection( $data );

			case "db2":
				include_once getabspath("connections/DB2Connection.php");
				return new DB2Connection( $data );

			case "informix":
				include_once getabspath("connections/InformixConnection.php");
				return new InformixConnection( $data );

			case "sqlite":
				include_once getabspath("connections/SQLite3Connection.php");
				return new SQLite3Connection( $data );
		}
	}
	
	/**
	 * Set the data representing the project's 
	 * db connection properties
	 */	 
	protected function _setConnectionsData()
	{
		// content of this function can be modified on demo account
		// variable names $data and $connectionsData are important

		$connectionsData = array();
		
		$data = array();
		$data["dbType"] = 0;
		$data["connId"] = "1188mall_at_localhost";
		$data["connName"] = "skyfreight at localhost";
		$data["connStringType"] = "mysql";
		$data["connectionString"] = "mysql;localhost;root;;;skyfreight;;1"; //currently unused
		
		$this->_connectionsIdByName["skyfreight at localhost"] = "1188mall_at_localhost";
		
		$data["connInfo"] = array();
		$data["ODBCUID"] = "root";
		$data["ODBCPWD"] = "";
		$data["leftWrap"] = "`";
		$data["rightWrap"] = "`";
		
		$data["DBPath"] = "db"; //currently unused	
		$data["useServerMapPath"] = 1; //currently unused
		
$host="192.168.1.224";
$user="root";
$pwd="birdistheword978234";
$port="";
//$sys_dbname="erp_cake2go3";

$sys_dbname="skyfreightv3";
$sys_dbname="skyfreightv3";
 
$_SESSION["dbname"] =$sys_dbname;

$_SESSION["ea"] = $sys_dbname;

$_SESSION["path"] = "http://" . $_SERVER['SERVER_NAME']  . ":8080/birt/";
$data["connInfo"][0] = $host;
$data["connInfo"][1] = $user;
$data["connInfo"][2] = $pwd;
$data["connInfo"][3] = $port;
$data["connInfo"][4] = $sys_dbname;
;
		$connectionsData["1188mall_at_localhost"] = $data;
		$this->_connectionsData = $connectionsData;
	}
	
	/**
	 * Set the data representing the correspondence between 
	 * the project's table names and db connections
	 */	 
	protected function _setTablesConnectionIds()
	{
		$connectionsIds = array();
		$connectionsIds["electricity"] = "1188mall_at_localhost";
		$connectionsIds["electricity_detail"] = "1188mall_at_localhost";
		$connectionsIds["gas"] = "1188mall_at_localhost";
		$connectionsIds["gas_detail"] = "1188mall_at_localhost";
		$connectionsIds["water"] = "1188mall_at_localhost";
		$connectionsIds["water_detail"] = "1188mall_at_localhost";
		$connectionsIds["rent"] = "1188mall_at_localhost";
		$connectionsIds["stall"] = "1188mall_at_localhost";
		$connectionsIds["system_user"] = "1188mall_at_localhost";
		$connectionsIds["cusa"] = "1188mall_at_localhost";
		$connectionsIds["cusa_detail"] = "1188mall_at_localhost";
		$connectionsIds["floor"] = "1188mall_at_localhost";
		$connectionsIds["payment_detail_water"] = "1188mall_at_localhost";
		$connectionsIds["payment_detail_gas"] = "1188mall_at_localhost";
		$connectionsIds["payment_detail_electricity"] = "1188mall_at_localhost";
		$connectionsIds["payment_detail_cusa"] = "1188mall_at_localhost";
		$connectionsIds["rent_detail"] = "1188mall_at_localhost";
		$connectionsIds["reservation_type"] = "1188mall_at_localhost";
		$connectionsIds["cusa_pdc_detail"] = "1188mall_at_localhost";
		$connectionsIds["rent_pdc_detail"] = "1188mall_at_localhost";
		$connectionsIds["cusa_pdc_for_deposit"] = "1188mall_at_localhost";
		$connectionsIds["rent_pdc_for_deposit"] = "1188mall_at_localhost";
		$connectionsIds["CUSA_for_billing"] = "1188mall_at_localhost";
		$connectionsIds["electricity_for_billing"] = "1188mall_at_localhost";
		$connectionsIds["Gas_for_billing"] = "1188mall_at_localhost";
		$connectionsIds["Rent_for_billing"] = "1188mall_at_localhost";
		$connectionsIds["Water_for_billing"] = "1188mall_at_localhost";
		$connectionsIds["available_stall"] = "1188mall_at_localhost";
		$connectionsIds["rent_payment"] = "1188mall_at_localhost";
		$connectionsIds["CUSA_payment"] = "1188mall_at_localhost";
		$connectionsIds["Gas_payment"] = "1188mall_at_localhost";
		$connectionsIds["Electricity_payment"] = "1188mall_at_localhost";
		$connectionsIds["Water_payment"] = "1188mall_at_localhost";
		$connectionsIds["payment_detail_rent"] = "1188mall_at_localhost";
		$connectionsIds["payment_form"] = "1188mall_at_localhost";
		$connectionsIds["Rent_Summary"] = "1188mall_at_localhost";
		$connectionsIds["Water_Readings"] = "1188mall_at_localhost";
		$connectionsIds["Gas_Readings"] = "1188mall_at_localhost";
		$connectionsIds["Electricity_Readings"] = "1188mall_at_localhost";
		$connectionsIds["rent_contract"] = "1188mall_at_localhost";
		$connectionsIds["Expiring_contract"] = "1188mall_at_localhost";
		$connectionsIds["chart_of_accounts"] = "1188mall_at_localhost";
		$connectionsIds["area"] = "1188mall_at_localhost";
		$connectionsIds["credit_terms"] = "1188mall_at_localhost";
		$connectionsIds["ewt"] = "1188mall_at_localhost";
		$connectionsIds["discounts"] = "1188mall_at_localhost";
		$connectionsIds["electricity_charges"] = "1188mall_at_localhost";
		$connectionsIds["stall_tenant"] = "1188mall_at_localhost";
		$connectionsIds["journal_entry_rent_contract"] = "1188mall_at_localhost";
		$connectionsIds["journal_entry_rent_amortization"] = "1188mall_at_localhost";
		$connectionsIds["tenant"] = "1188mall_at_localhost";
		$connectionsIds["journal_entry_electricity_billing"] = "1188mall_at_localhost";
		$connectionsIds["sales_rental"] = "1188mall_at_localhost";
		$connectionsIds["sales_rental_detail"] = "1188mall_at_localhost";
		$connectionsIds["item"] = "1188mall_at_localhost";
		$connectionsIds["vat_type"] = "1188mall_at_localhost";
		$connectionsIds["journal_entry_billing_rental"] = "1188mall_at_localhost";
		$connectionsIds["cash_receipt_rental"] = "1188mall_at_localhost";
		$connectionsIds["cash_receipt_detail_rental"] = "1188mall_at_localhost";
		$connectionsIds["bal_cashreceipt_rental"] = "1188mall_at_localhost";
		$connectionsIds["journal_entry_rent_collection"] = "1188mall_at_localhost";
		$connectionsIds["bal_electricity"] = "1188mall_at_localhost";
		$connectionsIds["bal_gas"] = "1188mall_at_localhost";
		$connectionsIds["bal_rent"] = "1188mall_at_localhost";
		$connectionsIds["bal_water"] = "1188mall_at_localhost";
		$connectionsIds["cash_receipt_detail2_rental"] = "1188mall_at_localhost";
		$connectionsIds["cost_center"] = "1188mall_at_localhost";
		$connectionsIds["currency"] = "1188mall_at_localhost";
		$connectionsIds["statement_of_account_rental"] = "1188mall_at_localhost";
		$connectionsIds["bank"] = "1188mall_at_localhost";
		$this->_tablesConnectionIds = $connectionsIds;
	}
	
	/**
	 * Check if It's possible to add to one table's sql query 
	 * an sql subquery to another table.
	 * Access doesn't support subqueries from the same table as main.
	 * @param String dataSourceTName1
	 * @param String dataSourceTName2
	 * @return Boolean
	 */
	public function checkTablesSubqueriesSupport( $dataSourceTName1, $dataSourceTName2 )
	{
		$connId1 = $this->_tablesConnectionIds[ $dataSourceTName1 ];
		$connId2 = $this->_tablesConnectionIds[ $dataSourceTName2 ];
		
		if( $connId1 != $connId2 )
			return false;

		if( $this->_connectionsData[ $connId1 ]["dbType"] == nDATABASE_Access && $dataSourceTName1 == $dataSourceTName2 )
			return false;
			
		return true;	
	}
	
	/**
	 * Close db connections
	 * @destructor
	 */
	function __desctruct() 
	{
		foreach( $this->cache as $connection )
		{
			$connection->close();
		}
	}
}
?>